/** 
 *   DGPS - Dump GPS utility.
 *
 *   (c) Copyright 2007 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 *   TODO: ???
 *      *)
 *      *)
 **/


#define TEST_MODE

#ifndef NODAEMON
#  define DAEMON
#endif

#define _GNU_SOURCE

#include <errno.h> 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <getopt.h>
#include <limits.h>
#include <syslog.h>
#include <sys/types.h>
#include <sys/wait.h>

#include <my_stdio.h>

#include "version.h"
#include "gps_rs232.h"
#include "dgps_conf.h"
#include "dgps_data.h"

const char *name_definition = {"DGPS - Dump GPS data"};
const char *help_text[] ={
#       ifdef VERBOSE_MODE
        "  -v\t\t - verbose mode (incremental).",
#       endif
#       ifdef DAEMON
        "  -n, --nodaemon\t - no run as a daemon.",
#       endif
        "  --nolog\t - Not write events to syslog.",
        "  -V\t\t - show program version.",
        "  --help\t - this help !!!.",
	"  -f, --flash\t - Saves the current configuration in flash memory",
        ""
};


void close_log(void)
{
        SYSLOG(LOG_INFO, "Program terminated.");
}

void fireman(int s)
{ 
	int wstatus;

	while(wait3(&wstatus, WNOHANG, NULL) > 0);
}


void version(char *program_name)
{
        fprintf(stderr,"%s: ", name_definition);
        fprintf(stderr,"%s version %s\n", program_name, VERSION);
}


void usage(char *program_name)
{
        version(program_name);
        fprintf(stderr,
#               ifdef VERBOSE_MODE
 "Usage: \t%s [-vnhf][--nodaemon][--nolog][--flash][--help]\n",
#                else
 "Usage: \t%s [-nhf][--nodaemon][--nolog][--flash][--help]\n",
#               endif
                program_name);
}

void help()
{
        int i=0;

        while((char) *help_text[i] != 0)
                fprintf(stderr,"%s\n",help_text[i++]);
        fprintf(stderr,"\n");
}


int main_loop(int fd)
{
	struct data_line dl;
	struct epoch ep;
	int valid_epoch = 0;
	int sat_count = 0; /* two lines for each sattelite */

	while (dgps_get_line(fd, &dl) == 0) {
		if (dl.code == LEAD_CODE_XOBS) {
			if (line_is_epoch(dl.line)) {
				if(valid_epoch) {
					print_epoch(1, &ep);
					valid_epoch = 0;
				}
				if (dl.valid_flag) {
					if (parse_epoch_line( dl.line, &ep) ==
					    0) {
						valid_epoch = 1;
						sat_count = 0;
					}  
				}
			} else if(valid_epoch) {
				if (dl.valid_flag) {
					ep.sat_valid_list[(sat_count>>1)]++;
					memcpy(ep.sat[sat_count],
					       dl.line, dl.len+1);
				}
				sat_count++;
				if (sat_count >= (ep.num_sat << 1)) {
					print_epoch(1, &ep);
					valid_epoch = 0;
				}
			}
		} else if (dl.code == LEAD_CODE_XOHD) {
			printf("%s\n", dl.line);
		}
	}
	
	return 0;
}




int main(int argc, char *argv[]) 
{
	int fd;
	int pid;
	int i;
	int daemon_flag = 1;
	int flash_flag = 0;
	char *cp;
	char *program_name;
	int op;
	char device[_POSIX_PATH_MAX];
	int dev_force_flag  = 0;
	struct dgps_conf *gc= NULL;
	struct rinex_heder *rh = NULL;

	static struct option  const  longopts[] =
	{
		{"help", 0, 0, 'h'},
		{"verbose", 0, 0, 'v'},
		{"nodaemon", 0, 0, 'n'},
		{"flash", 0, 0, 'f'},
		{"device", 1, 0, 'd'},
		{"nolog", 0, 0, 201},
		{0,  0,  0,  0}
	};
	
	/* who is my name? */
        if ((cp = strrchr(argv[0], '/')) != NULL) program_name  = cp + 1;
        else program_name  = argv[0];

	syslog_flag = 1;
	opterr =  0;
	while  ((op = getopt_long(argc, argv, "vhnVd:f",longopts, 0)) != EOF)
		switch( op ){
		case 'v':
			verbose_flag = (verbose_flag << 1) | 1;
			break;
		case 'n':
                        daemon_flag = 0;
                        break;
		case 'f':
                        flash_flag = 1;
			daemon_flag = 0;
                        break;
		case 201:
                        syslog_flag = 0;
                        break;
		case 'd':
                        snprintf(device, _POSIX_PATH_MAX, "%s", optarg);
			dev_force_flag = 1;
                        break;
		case 'V':
                        version(program_name);
                        exit(-1);
                        break;
		case 'h':
			usage(program_name);
			help();
			exit(-1); 
		default:
			usage(program_name);
			exit(-1);
		}

	gc = dgps_get_conf("dgps.conf");
	if (dev_force_flag) {
		snprintf(gc->device, _POSIX_PATH_MAX, "%s", device);
	}

	
#ifdef DAEMON
	if (daemon_flag)
		pid = fork();
	else
		pid = 0;
#else
	pid = 0;
#endif
	if ( pid == 0 ) {
		if(syslog_flag){
			openlog(program_name, LOG_PID, LOG_DAEMON );
			atexit(close_log);
			SYSLOG(LOG_INFO, "Starting %s...", program_name);
		}
		if ( gc->set_rinex_flag ) /* read header file if defined */
			rh = rinex_heder_get_conf(gc->rinex_header);
		/* open gps: */
		if ((fd = hw_open(gc->device)) <= 0){
			SYSLOG(LOG_ERR,"can't open GPS");
			exit(1);
		}
		if (flash_flag) {
			printf("Un logging all.\n");
			while (set_unlogging(fd) != 0);
			printf("Reseting configuration");
			reset_conf(fd);
			for(i=0;i<10;i++){
				printf(".");
				fflush(stdout);
				sleep(1);
			}
			printf("\n");
			if (rh != NULL) {
				printf("Seting RINEX heder information.\n");
				set_rinex_heder(fd, rh);
			} else 
				SYSLOG(LOG_ERR,"RINEX heder: undefined");	
			printf("Seting logging mode.\n");
			set_logging(fd, gc);
			printf("Save all in flash (non-volatile) memory.\n");
			save_flash(fd);
		} else {
#ifndef TEST_MODE
			while (set_unlogging(fd) != 0);
			if (rh != NULL) 
				set_rinex_heder(fd, rh);
			else 
				SYSLOG(LOG_ERR,"RINEX heder: undefined");	
			set_logging(fd, gc);
			/* FIXME: main loop aca */
#endif
			main_loop(fd);
		}

		hw_close( fd );
		free_rinex_heder(rh);
		exit(0);
	}

	return 0;

}




